@@ -29,7 +29,6 @@
 #include <QSvgRenderer>
 #include <QGraphicsRectItem>
 #include <QGraphicsPixmapItem>
-#include <QPropertyAnimation>
 #include <QImage>
 #include <QPainter>
 #include <QStyleOptionGraphicsItem>
@@ -97,6 +96,9 @@ public:
     QPoint mapToCell() const;
     QPoint mapFromCell(const QPoint &cell) const;
 
+Q_SIGNALS:
+    void pieceClicked(MPiece *piece);
+
 public Q_SLOTS:
     void select();
     void deSelect();
@@ -109,7 +111,6 @@ public Q_SLOTS:
     void rotate0();
     void rotate180();
     void rotate(bool flip);
-    void onRotationFinished();
 
     void toggleRotations();
 
@@ -120,6 +121,10 @@ protected:
     virtual void initializeEffects();
     virtual void applyRenderer(QSvgRenderer &renderer, int pieceSize);
 
+    // handle the mousePressEvent directly because Qt fucked up scene event
+    // propagation, yeah!
+    virtual void mousePressEvent(QGraphicsSceneMouseEvent *event);
+
     MColour colour;
     MType type;
     int xDim;
@@ -136,16 +141,10 @@ protected:
     QGraphicsPixmapItem *ghost;
     QGraphicsPixmapItem *dropShadow;
 
-    // Flipping a piece, animated.
-    bool rotated;
-    QPropertyAnimation *rotationAnimForward;
-    QPropertyAnimation *rotationAnimForwardCcw;
-    QPropertyAnimation *rotationAnimBackward;
-    QPropertyAnimation *rotationAnimBackwardCcw;
-
     // Fade out the ghost piece ... somehow, QPropertyAnimations did not work
     QTimeLine *ghostFadeOutTimer;
 
+    bool rotated;
     bool enableRotations;
 };
 
